#pragma once

#include "CoreMinimal.h"
#include "RendererInterface.h"
#include "ISpectatorScreenController.h"
#include "HeadMountedDisplayTypes.h"
#include "UObject/WeakObjectPtrTemplates.h"
#include "TiltFiveXRBase.h"

class FTiltFiveXRBase;

DECLARE_DELEGATE_FiveParams(FSpectatorScreenRenderDelegate, FRHICommandListImmediate& /* RHICmdList */, FTexture2DRHIRef /* TargetTexture */, FTexture2DRHIRef /* EyeTexture */, FTexture2DRHIRef /* OtherTexture */, FVector2D /* WindowSize */);

/**
 *	Default implementation of spectator screen controller.
 *
 */
class TILTFIVE_API TiltFiveSpectatorController : 
	public ISpectatorScreenController, 
	public TSharedFromThis<TiltFiveSpectatorController, ESPMode::ThreadSafe>
{
public:
	TiltFiveSpectatorController(FTiltFiveXRBase* InHMDDevice);
	virtual ~TiltFiveSpectatorController() {}

	// ISpectatorScreenController
	virtual ESpectatorScreenMode GetSpectatorScreenMode() const override;
	virtual void SetSpectatorScreenMode(ESpectatorScreenMode Mode) override;
	virtual void SetSpectatorScreenTexture(UTexture* InTexture) override;
	virtual UTexture* GetSpectatorScreenTexture() const override;
	virtual void SetSpectatorScreenModeTexturePlusEyeLayout(const FSpectatorScreenModeTexturePlusEyeLayout& Layout) override;
	virtual void QueueDebugCanvasLayerID(int32 LayerID) override;

	FSpectatorScreenRenderDelegate* GetSpectatorScreenRenderDelegate_RenderThread();


	// Implementation methods called by HMD
	virtual void BeginRenderViewFamily();
	virtual void UpdateSpectatorScreenMode_RenderThread();
	virtual void RenderSpectatorScreen_RenderThread(FRHICommandListImmediate& RHICmdList, FRHITexture2D* BackBuffer, FTexture2DRHIRef SrcTexture, FVector2D WindowSize);
	virtual void RenderSpectatorScreen_RenderThread(FRHICommandListImmediate& RHICmdList, FRHITexture2D* BackBuffer, FTexture2DRHIRef SrcTexture, FTexture2DRHIRef LayersTexture, FVector2D WindowSize);

protected:
	friend struct FRHISetSpectatorScreenTexture;
	virtual void SetSpectatorScreenTextureRenderCommand(UTexture* SrcTexture);
	virtual void SetSpectatorScreenTexture_RenderThread(FTexture2DRHIRef& InTexture);

	friend struct FRHISetSpectatorScreenModeTexturePlusEyeLayout;
	virtual void SetSpectatorScreenModeTexturePlusEyeLayoutRenderCommand(const FSpectatorScreenModeTexturePlusEyeLayout& Layout);
	virtual void SetSpectatorScreenModeTexturePlusEyeLayout_RenderThread(const FSpectatorScreenModeTexturePlusEyeLayout& Layout);

	virtual FIntRect GetFullFlatEyeRect_RenderThread(FTexture2DRHIRef EyeTexture);

	virtual void RenderSpectatorModeSingleEyeLetterboxed(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeUndistorted(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeDistorted(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeSingleEye(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeTexture(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeMirrorAndTexture(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);
	virtual void RenderSpectatorModeSingleEyeCroppedToFill(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, FTexture2DRHIRef EyeTexture, FTexture2DRHIRef OtherTexture, FVector2D WindowSize);

	virtual FRHITexture2D* GetFallbackRHITexture() const;

	mutable FCriticalSection NewSpectatorScreenModeLock;
	ESpectatorScreenMode NewSpectatorScreenMode = ESpectatorScreenMode::SingleEyeCroppedToFill;
	TWeakObjectPtr<UTexture> SpectatorScreenTexture;

	ESpectatorScreenMode SpectatorScreenMode_RenderThread = ESpectatorScreenMode::Disabled;
	FTexture2DRHIRef SpectatorScreenTexture_RenderThread;
	FSpectatorScreenModeTexturePlusEyeLayout SpectatorScreenModeTexturePlusEyeLayout_RenderThread;
	FSpectatorScreenRenderDelegate SpectatorScreenDelegate_RenderThread;
	TArray<int32> DebugCanvasLayerIDs;

		static FIntRect GetEyeCroppedToFitRect(FVector2D EyeCenterPoint, const FIntRect& EyeRect, const FIntRect& TargetRect);
		static FIntRect GetLetterboxedDestRect(const FIntRect& SrcRect, const FIntRect& TargetRect);

private:
	void CopyEmulatedLayers(FRHICommandListImmediate& RHICmdList, FTexture2DRHIRef TargetTexture, const FIntRect SrcRect, const FIntRect DstRect);

	FTiltFiveXRBase* HMDDevice;
	// Face locked stereo layers are composited to a single texture which has to be copied over to the spectator screen.
	FTexture2DRHIRef StereoLayersTexture;
};
